#include <FreqCount.h>
//
#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#define LCDStrLen 16
LiquidCrystal_I2C lcd(0x27, LCDStrLen, 2);// Устанавливаем параметры дисплея
//
#include <SimpleRotary.h>
SimpleRotary rotary (3, 4, 2);
#define Key5VPin 6
byte rDir, lastDir = 0;
byte rLBtn, rBtn = 0;
//
#define latchPin 11
#define clockPin 10
#define dataPin 12

// (Real lowest freq 108kHz (11))
// (Real highest freq 162kHz (187))

//LW
#define minFreqDivLW 16 // 153kHz (16)
#define maxFreqDivLW 30 // 279kHz (30)

//MW
#define minFreqDivMW 58 // 531kHz (58)
#define maxFreqDivMW 177 // 1602kHz (177)

#define startFreqDiv 16 // Start freqency devider (16-30,58-177)

#define SynStandby 0
#define SynCheck 1
#define SynReady 2
#define SynRun 3
#define SynError 4

#define OutEnPin 7

#define MsgError 0
#define MsgStandby 1
#define MsgCheck 2
#define MsgReady 3
#define MsgRun 4

#define MesurePreScale 1
#define MesureFreqError 4.0 // +/-  
//#define SkipFreqCheck
//#define SkipRunFreqCheck

static byte FreqDiv, FreqDivOld = startFreqDiv;
static byte SynState = SynStandby;

String LCDStr;

void setup() {
  FreqDiv = startFreqDiv;
  SynState = SynStandby;
  pinMode (Key5VPin, OUTPUT);
  digitalWrite (Key5VPin, HIGH);

  pinMode (OutEnPin, OUTPUT);
  digitalWrite (OutEnPin, LOW);

  pinMode (latchPin, OUTPUT);
  pinMode (dataPin, OUTPUT);
  pinMode (clockPin, OUTPUT);
//
  set_registr (FreqDiv);
  digitalWrite (OutEnPin, LOW);
//
  rotary.setDebounceDelay(3);
  rotary.setErrorDelay(150);
//
  Wire.begin ();
  Wire.setClock (1000);
//
  lcd.init ();
  lcd.clear ();
  lcd.backlight ();// Включаем подсветку дисплея
  lcd.setCursor (0, 0);
  lcd.print (F ("LW-MW Synt v.0.2"));
  lcd.setCursor (0, 1);
  lcd.print (__DATE__);
  // lcd.setCursor (12, 1);
  // lcd.print (__TIME__);
//
  delay (3000);
//
  // Serial.begin (9600);
//
  FreqCount.begin (1000*MesurePreScale);
}

void set_registr (byte RegData) {
    //устанавливаем LOW на latchPin пока не окончена передача байта
    digitalWrite (latchPin, LOW);
    shiftOut (dataPin, clockPin, MSBFIRST, RegData);  
    //устанавливаем HIGH на latchPin, чтобы проинформировать регистр, что передача окончена.
    digitalWrite (latchPin, HIGH);   
}

int digitalFreq (byte FreqDiv) {
  return ((FreqDiv+1)*9);
}

void print_status (byte FreqDiv, byte Msg) {
  lcd.clear ();
  lcd.setCursor (0, 0);
  if ((FreqDiv >= minFreqDivLW) && (FreqDiv <= maxFreqDivLW)) { // LW band
    LCDStr = "LW";
    LCDStr += String (digitalFreq (FreqDiv))+"kHz";
  } else if ((FreqDiv >= minFreqDivMW) && (FreqDiv <= maxFreqDivMW)) { // MW band
    LCDStr = "MW";
    LCDStr += String (digitalFreq (FreqDiv))+"kHz";
  }
  lcd.print (LCDStr);
  lcd.setCursor (0, 1);
  if (Msg == MsgError) {
    lcd.print ("Error");
  } else if (Msg == MsgStandby) {
    lcd.print ("Standby");
  } else if (Msg == MsgReady) {
    lcd.print ("Ready");
  } else if (Msg == MsgCheck) {
    lcd.print ("Check");
  } else if (Msg == MsgRun) {
    lcd.print ("Run");
  } else {
    lcd.print ("Unknow");
  }
}

void loop() {
  print_status (FreqDiv, MsgStandby);

  while (1) {
    rDir = rotary.rotate();
    rBtn = rotary.pushType(250);;
  
    if (SynState == SynStandby) {
      if ((FreqDiv >= minFreqDivLW) && (FreqDiv <= maxFreqDivLW)) { // LW band
        if (rDir == 1) { // Rotate - CW
          if (FreqDiv > minFreqDivLW) {
            FreqDiv--;
          } else {
            FreqDiv = maxFreqDivMW;
          }
        } else if (rDir == 2) { // Rotate - CCW
          if (FreqDiv < maxFreqDivLW) {
            FreqDiv++;
          } else {
            FreqDiv = minFreqDivMW;
          }
        }
      } else if ((FreqDiv >= minFreqDivMW) && (FreqDiv <= maxFreqDivMW)) { // MW band
        if (rDir == 1) { // Rotate - CW
          if (FreqDiv > minFreqDivMW) {
            FreqDiv--;
          } else {
            FreqDiv = maxFreqDivLW;
          }
        } else if (rDir == 2) { // Rotate - CCW
          if (FreqDiv < maxFreqDivMW) {
            FreqDiv++;
          } else {
            FreqDiv = minFreqDivLW;
          }
        }
      } else { // Error
        print_status (FreqDiv, MsgError);
        digitalWrite (OutEnPin, LOW);
      }
      
      if (FreqDiv != FreqDivOld) {
        FreqDivOld = FreqDiv;
        print_status (FreqDiv, MsgStandby);
        set_registr (FreqDiv);
      }
    }
    
    if ( rBtn == 1 ) { // Button - Short
      SynState = SynStandby;
      print_status (FreqDiv, MsgStandby);
      digitalWrite (OutEnPin, LOW);
    } else if (rBtn == 2) { // Button - Long
      if (SynState == SynStandby) {
        SynState = SynCheck;
        print_status (FreqDiv, MsgCheck);
      } else if (SynState == SynReady) {
        SynState = SynRun;
        print_status (FreqDiv, MsgRun);
        digitalWrite (OutEnPin, HIGH);
      }
    }
//
    #ifndef SkipFreqCheck
      #ifndef SkipRunFreqCheck
        if ((SynState == SynCheck) || (SynState == SynReady) || (SynState == SynRun))  {
      #else
        if ((SynState == SynCheck) || (SynState == SynReady))  {
      #endif
        if (FreqCount.available()) { 
          unsigned long f = FreqCount.read(); // Dummy read
        }

        delay (500); // WHY ?

        if (FreqCount.available()) {
          float MesureFreq = (float)FreqCount.read()/(1000*MesurePreScale);
          if ((MesureFreq <= (float)(digitalFreq (FreqDiv) + MesureFreqError)) && (MesureFreq >= (float)(digitalFreq (FreqDiv) - MesureFreqError))) {
            if (SynState != SynRun) {
              print_status (FreqDiv, MsgReady);
              // lcd.setCursor (10,0);
              // lcd.print (String ((float)MesureFreq));
              SynState = SynReady;
            } else if (SynState == SynRun) {
              print_status (FreqDiv, MsgRun);
              // lcd.setCursor (10,0);
              // lcd.print (String ((float)MesureFreq));
              SynState = SynRun;
            } 
          } else {
            print_status (FreqDiv, MsgError);
            lcd.setCursor (10,0);
            lcd.print (String ((float)MesureFreq));
            SynState = SynError;
            digitalWrite (OutEnPin, LOW);
          }
        }
      }
    #else
      if (SynState == SynCheck) {
        print_status (FreqDiv, MsgReady);
        SynState = SynReady;
      }
    #endif
  }

}
